'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

require('core-js/modules/es7.object.values');

require('core-js/modules/es7.object.entries');

require('core-js/modules/es7.object.get-own-property-descriptors');

require('core-js/modules/es7.string.pad-start');

require('core-js/modules/es7.string.pad-end');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Module to handle application logging.
 */
/**
 * Copyright 2014 Skytap Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License")
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

const Logger = {

  levels: ['debug', 'info', 'warn', 'error'],

  loggers: [],

  /**
   * Logger.initialize()
   *
   * Initialize the logger.
   */
  initialize(options) {
    this.options = options;

    if (options.loggers && Array.isArray(options.loggers) && options.loggers.length > 0) {
      this.loggers = this.loadLoggers(options.loggers);
    }

    this.addLevelHandlers();
  },

  /**
   * Logger.profile(name, start)
   * - name (String)
   * - start (Integer): Start time in milliseconds
   *
   * Output a debug log of the number of milliseconds it took to complete a task.
   */
  profile(name, start, end, contextId) {
    const endTime = typeof end === 'undefined' ? Date.now() : end;
    this.log('debug', `Performance: ${name} took ${endTime - start}ms`, null, contextId);
  },

  debug() {},

  error() {},

  /**
   * Logger.addLevelHandlers()
   */
  addLevelHandlers() {
    this.levels.forEach(level => {
      this[level] = (...args) => {
        args.unshift(level);
        this.log(...args);
      };
    });
  },

  /**
   * Logger.loadLoggers(loggers) -> Array
   * - loggers (Array): Array of logger names
   */
  loadLoggers(loggers) {
    return loggers.map(logger => this.loadLogger(logger));
  },

  /**
   * Logger.loadLogger(file) -> Object
   * - file (Mixed): Either the name of a logger or a logger class
   */
  loadLogger(file) {
    let LoggerModule;

    if (typeof file === 'function') {
      LoggerModule = file;
    } else {
      const loggerPath = _path2.default.join(this.options.basePath, 'lib', 'loggers');
      // eslint-disable-next-line import/no-dynamic-require, global-require
      LoggerModule = require(_path2.default.join(loggerPath, `${file}_logger`));
      LoggerModule = LoggerModule.default ? LoggerModule.default : LoggerModule;
    }

    return new LoggerModule(this.levels);
  },

  /**
   * Logger.log()
   *
   * Log a message using the specified loggers.
   */
  log(...args) {
    this.loggers.forEach(logger => logger.log(...args));
  }
};

exports.default = Logger;