'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Path = exports.Logger = exports.FileSystem = exports.Environment = exports.Controller = exports.Config = undefined;

require('core-js/modules/es7.object.values');

require('core-js/modules/es7.object.entries');

require('core-js/modules/es7.object.get-own-property-descriptors');

require('core-js/modules/es7.string.pad-start');

require('core-js/modules/es7.string.pad-end');

var _cluster = require('cluster');

var _cluster2 = _interopRequireDefault(_cluster);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _cookieParser = require('cookie-parser');

var _cookieParser2 = _interopRequireDefault(_cookieParser);

var _errorhandler = require('errorhandler');

var _errorhandler2 = _interopRequireDefault(_errorhandler);

var _compression = require('compression');

var _compression2 = _interopRequireDefault(_compression);

var _serveFavicon = require('serve-favicon');

var _serveFavicon2 = _interopRequireDefault(_serveFavicon);

var _bodyParser = require('body-parser');

var _bodyParser2 = _interopRequireDefault(_bodyParser);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _backhoe = require('backhoe');

var _backhoe2 = _interopRequireDefault(_backhoe);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _controller = require('./controller');

var _controller2 = _interopRequireDefault(_controller);

var _environment = require('./environment');

var _environment2 = _interopRequireDefault(_environment);

var _router = require('./router');

var _router2 = _interopRequireDefault(_router);

var _logger = require('./logger');

var _logger2 = _interopRequireDefault(_logger);

var _template = require('./template');

var _template2 = _interopRequireDefault(_template);

var _filter = require('./filter');

var _filter2 = _interopRequireDefault(_filter);

var _path3 = require('./path');

var _path4 = _interopRequireDefault(_path3);

var _filesystem = require('./filesystem');

var _filesystem2 = _interopRequireDefault(_filesystem);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Module for creating an HTTP server.
 */
class Minor {
  constructor(options) {
    this.app = (0, _express2.default)();
    this.options = options;
    this.closing = false;

    // bind functions
    this.listen = this.listen.bind(this);
    this.initialize = this.initialize.bind(this);

    if (!this.options.instance) {
      this.options.instance = 1;
    }

    if (!this.options.controllerTimeout) {
      // default timeout 3 minutes
      this.options.controllerTimeout = 180000;
    }

    if (!this.options.hostname) {
      this.options.hostname = '0.0.0.0';
    }

    this.initializeEnvironment();
    this.initializeErrorHandling();
  }

  /**
   * Minor.initialize()
   */
  initialize() {
    const start = Date.now();

    this.start = start;

    this.app.use((0, _cookieParser2.default)());

    const defaultPromise = this.initializeDefaultConfiguration();

    const environmentPromise = this.isProduction() ? this.initializeProductionConfig() : this.initializeDevelopmentConfig();

    // listen for a shutdown message
    process.on('message', this.handleMessage.bind(this));

    return _bluebird2.default.all([defaultPromise, environmentPromise]).then(() => {
      _logger2.default.profile('Initialize MinorJS', start);
    });
  }

  /**
   * Minor.listen() -> Object
   */
  listen() {
    const start = Date.now();

    return _router2.default.load(this.app, `${this.options.basePath}/lib/controllers`, this.options).then(() => {
      _logger2.default.profile('Load controllers', start);

      const port = this.getPort();
      this.server = this.app.listen(port, this.options.hostname);

      _logger2.default.profile('Load MinorJS', this.start);

      _logger2.default.info(`MinorJS HTTP server listening on ${this.options.hostname}:${port}`);
    }).catch(error => {
      // eslint-disable-next-line no-console
      console.error('Error loading controllers and registering routes.\n\n', error.stack);
      process.exit(1);
    });
  }

  /**
   * Minor.getEnvironment() -> String
   */
  getEnvironment() {
    return typeof process.env.NODE_ENV === 'undefined' ? 'development' : process.env.NODE_ENV;
  }

  /**
   * Minor.getPort()
   */
  getPort() {
    if (this.options.port) {
      return this.options.port;
    }

    return _config2.default.get('port');
  }

  /**
   * Minor.initializeDefaultConfiguration()
   */
  initializeDefaultConfiguration() {
    const start = Date.now();

    return _bluebird2.default.resolve().then(() => {
      this.app.use((0, _compression2.default)());

      this.app.set('views', `${this.options.basePath}/lib/templates`);

      this.app.locals.escapeAttributes = true;

      if (Array.isArray(this.options.templatePlugins)) {
        this.options.templatePlugins.forEach(plugin => plugin.register(this.app));
      }
    }).then(() =>
    // load all the template mixins
    _template2.default.loadMixins(`${this.options.basePath}/lib/template_mixins`)).then(() =>
    // load all the filters
    _filter2.default.load(`${this.options.basePath}/lib/filters`)).then(() => {
      const faviconPath = `${this.options.basePath}/public/favicon.ico`;

      if (_fs2.default.existsSync(faviconPath)) {
        this.app.use((0, _serveFavicon2.default)(faviconPath));
      }

      // parse application/x-www-form-urlencoded
      this.app.use(_bodyParser2.default.urlencoded({ extended: false }));

      // parse application/json
      this.app.use(_bodyParser2.default.json());
    }).then(() => {
      _logger2.default.profile('Initialize default configuration', start);
    });
  }

  /**
   * Minor.initializeDevelopmentConfig()
   */
  initializeDevelopmentConfig() {
    const start = Date.now();
    const noCacheDefaults = ['lib/controllers', 'lib/controller_helpers', 'lib/views', 'lib/templates'];

    return _bluebird2.default.resolve().then(() => {
      const noCachePaths = this.options.noCache || noCacheDefaults;
      _backhoe2.default.noCache(_environment2.default.getBasePath(), noCachePaths);

      // pretty print the HTML
      this.app.locals.uglify = false;

      this.app.use((0, _errorhandler2.default)({
        dumpExceptions: true,
        showStack: true
      }));

      return this.loadMiddleware('development');
    }).then(() => {
      _logger2.default.profile('Initialize development configuration', start);
    });
  }

  /**
   * Minor.initializeProductionConfig()
   */
  initializeProductionConfig() {
    const start = Date.now();

    return _bluebird2.default.resolve().then(() => {
      this.app.locals.uglify = true;
      return this.loadMiddleware('production');
    }).then(() => {
      _logger2.default.profile('Initialize production configuration', start);
    });
  }

  /**
   * Minor.initializeEnvironment()
   */
  initializeEnvironment() {
    // Initialize the environment and configs
    _environment2.default.initialize({
      basePath: this.options.basePath,
      environment: this.app.get('env'),
      instance: this.options.instance,
      loggers: this.options.loggers || [],
      contextName: this.options.contextName
    });
  }

  /**
   * Minor.initializeErrorHandling()
   */
  initializeErrorHandling() {
    process.on('uncaughtException', error => {
      // eslint-disable-next-line no-console
      console.log(error.stack);

      _logger2.default.error(error.stack);

      if (_environment2.default.isWorker()) {
        _cluster2.default.worker.disconnect();
      } else {
        process.exit(1);
      }
    });

    this.app.on('error', error => {
      _logger2.default.error(error.stack);
    });
  }

  /**
   * Minor.isProduction() -> Boolean
   */
  isProduction() {
    return this.getEnvironment() === 'production';
  }

  /**
   * Minor.loadMiddleware()
   *
   * Load and register middleware.
   */
  loadMiddleware(environment) {
    const startMiddleware = Date.now();
    const middlewarePath = _path2.default.join(this.options.basePath, 'lib', 'middleware');

    if (!this.options.middleware || !this.options.middleware[environment]) {
      return _bluebird2.default.resolve();
    }

    this.options.middleware[environment].forEach(file => {
      const start = Date.now();
      // eslint-disable-next-line import/no-dynamic-require, global-require
      let middleware = require(_path2.default.join(middlewarePath, file));
      middleware = middleware.default ? middleware.default : middleware;
      middleware.process(this.app);
      _logger2.default.profile(`Load middleware ${file}`, start);
    });

    _logger2.default.profile('Load middleware', startMiddleware);

    return _bluebird2.default.resolve();
  }

  /**
   * Minor.handleMessage(message)
   */
  handleMessage(message) {
    switch (message) {
      case 'shutdown':
        this.shutdown();
        break;
      default:
      // no op
    }
  }

  /**
   * Minor.shutdown()
   */
  shutdown() {
    if (this.closing === true) {
      // already shutting down the worker
      return;
    }

    this.closing = true;

    // stop accepting HTTP connections
    this.server.close(() => {
      // shut down the worker. the cluster manager will spawn a replacement worker.
      _cluster2.default.worker.disconnect();
    });
  }
}

exports.default = Minor; /**
                          * Copyright 2014 Skytap Inc.
                          *
                          * Licensed under the Apache License, Version 2.0 (the "License")
                          * you may not use this file except in compliance with the License.
                          * You may obtain a copy of the License at
                          *
                          *     http://www.apache.org/licenses/LICENSE-2.0
                          *
                          * Unless required by applicable law or agreed to in writing, software
                          * distributed under the License is distributed on an "AS IS" BASIS,
                          * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
                          * See the License for the specific language governing permissions and
                          * limitations under the License.
                          */

exports.Config = _config2.default;
exports.Controller = _controller2.default;
exports.Environment = _environment2.default;
exports.FileSystem = _filesystem2.default;
exports.Logger = _logger2.default;
exports.Path = _path4.default;